'''
% Cox proportional hazards regression with Robust estimator
% This program provides interface of Cox proportional hazards regression for evaluating the effect of several risk factors such as sex, age, and weight on survival on survival.
%
% This program requires R, rpy, coxrobust package for R
%    R ==> http://www.r-project.org/
%    rpy ==> http://rpy.sourceforge.net/
%    coxrobust ==> http://cran.r-project.org/web/packages/coxrobust/index.html
%
% Last Update: 2011.06.03
% By Jae-Seong Yang, destine@postech.ac.kr
'''

from rpy import *
from CoxRegression import ParseCoxrData

class CoxResult:
    def __init__( self ):
        self.keys = []

        self.wald = []
        self.features = {}

        self.ewald = []
        self.efeatures = {}
   
def CoxrRobustResult( result, keys ):
    print ""
    print "Robust estimator"
    print "\t".join( ["","coef","exp(coef)","se(coef)","p"] )
    index = 0
    
    aResult = CoxResult()
    aResult.keys = keys

    for key in keys:
        coef = result[ 'coefficients' ][key]
        var = result["var"][index][index]
        se_coef = math.sqrt( var )
        z = coef / se_coef
        p_val = min( r.pnorm( z ), 1-r.pnorm( z ) ) * 2
        
        exp_coef = "%0.3f" % math.exp( coef )
        coef = "%0.2e" % coef
        se_coef = "%0.6f" % se_coef
        p_val = "%0.6f" % p_val
        print "\t".join( [ key, coef, exp_coef, se_coef, p_val ] )

        aResult.efeatures[ key ] = [ key, coef, exp_coef, se_coef, p_val ] 

        index += 1

    df = len( keys )
    ewald_test = result["ewald.test"]
    p = (1-r.pchisq( ewald_test, df ))

    print ""
    print "Extended Wald test=%0.1f on %d df, p=%0.6f" % ( ewald_test, df, p )

    aResult.wald = [ ewald_test, df, p ]  
    return aResult

def TestCox():
    r.library( "coxrobust" )
    result = r.coxr( r("Surv(time, status) ~ age + sex + ph.karno + meal.cal + wt.loss"), r("data = lung"))
    coxPLEResult = CoxrPLEResult( result, ["age","sex","ph.karno","meal.cal","wt.loss"] )
    coxRobustResult = CoxrRobustResult( result, ["age","sex","ph.karno","meal.cal","wt.loss"] )

    return ( coxPLEResult, coxRobustResult )

def RunCoxRegression( coxr_data, keys ):
    r.library( "coxrobust" )
    model = "Surv(time, status) ~"
    for key in keys:
        model += " + %s" % key

    result = r.coxr( r( model ), data = coxr_data )
    coxRobustResult = CoxrRobustResult( result, keys )

    return coxRobustResult



if __name__ == "__main__":
    [ coxr_data, keys ] = ParseCoxrData( "coxr_sample.txt" )
    RunCoxRegression( coxr_data, keys )

